#include "editor.h"
#include "highlighter.h"
#include "dialog.h"

Editor::Editor(QWidget *parent) : QMainWindow(parent), Ui::Editor()
{
    setupUi(this);
    setWindowIcon(QIcon(":/icon.png"));
    setWindowTitle("Document Editor");

    m_toolBar->setWindowTitle("Панель инструментов");
    m_toolBar->addAction(QIcon(":/new_icon.png"), "New", m_newAction, SLOT(trigger()));
    m_toolBar->addAction(QIcon(":/open_icon.png"), "Open", m_openAction, SLOT(trigger()));
    m_toolBar->addAction(QIcon(":/save_icon.png"), "Save", m_saveAction, SLOT(trigger()));
    m_toolBar->addAction(QIcon(":/saveall_icon.png"), "Save all", m_saveAllAction, SLOT(trigger()));

    m_fileModel = new QFileSystemModel(this);
    m_fileModel->setRootPath(QDir::rootPath());
    m_explorerTreeView->setModel(m_fileModel);
    connect(m_explorerTreeView, SIGNAL(doubleClicked(QModelIndex)), SLOT(slotExplorerOpen(QModelIndex)));

    m_list = new QStringList;
    m_listModel = new QStringListModel(this);
    m_listModel->setStringList(*m_list);
    m_viewerListView->setModel(m_listModel);

    connect(m_openAction, SIGNAL(triggered(bool)), SLOT(slotOpen()));
    connect(m_newAction, SIGNAL(triggered(bool)), SLOT(slotNew()));
    connect(m_saveAction, SIGNAL(triggered(bool)), SLOT(slotSave()));
    connect(m_saveAsAction, SIGNAL(triggered(bool)), SLOT(slotSaveAs()));
    connect(m_saveAllAction, SIGNAL(triggered(bool)), SLOT(slotSaveAll()));
    connect(m_closeAction, SIGNAL(triggered(bool)), SLOT(slotClose()));
    connect(m_closeAllAction, SIGNAL(triggered(bool)), SLOT(slotCloseAll()));
    connect(m_quitAction, SIGNAL(triggered(bool)), SLOT(slotQuit()));
    connect(m_cutAction, SIGNAL(triggered(bool)), SLOT(slotCut()));
    connect(m_copyAction, SIGNAL(triggered(bool)), SLOT(slotCopy()));
    connect(m_pasteAction, SIGNAL(triggered(bool)), SLOT(slotPase()));
    connect(m_deleteAction, SIGNAL(triggered(bool)), SLOT(slotDelete()));
    connect(m_selectAllAction, SIGNAL(triggered(bool)), SLOT(slotSelectAll()));
    connect(m_viewerAction, SIGNAL(toggled(bool)), SLOT(slotViewer(bool)));
    connect(m_explorerAction, SIGNAL(toggled(bool)), SLOT(slotExplorer(bool)));
    connect(m_aboutAction, SIGNAL(triggered(bool)), SLOT(slotAbout()));
    connect(m_tabWidget, SIGNAL(tabCloseRequested(int)), SLOT(slotCloseTab(int)));
    connect(m_explorerDock, SIGNAL(visibilityChanged(bool)), m_explorerAction, SLOT(setChecked(bool)));
    connect(m_viewerDock, SIGNAL(visibilityChanged(bool)), m_viewerAction, SLOT(setChecked(bool)));
}

void Editor::closeEvent(QCloseEvent* event)
{
     event->ignore();
    bool save_required = true;
    for(int i = 0; i < m_tabs.count(); i++)
        save_required &= m_tabs[i].m_saved;

    if(!save_required)
    {
        auto d = new Dialog(this, *m_list, m_tabs);
        connect(d, SIGNAL(signalSave()), this, SLOT(slotCloseAndSaveWindow()));
        connect(d, SIGNAL(signalDiscard()), this, SLOT(slotCloseWindow()));
        d->exec();
    }
    else if(save_required)
    {
        event->accept();
    }

}

void Editor::slotCloseAndSaveWindow()
{
    bypass = false;
    slotSaveAll();
    if(!bypass)
        QCoreApplication::quit();
}

void Editor::slotCloseWindow()
{
    QCoreApplication::quit();
}

void Editor::slotOpen()
{
    QFile *file = new QFile(QFileDialog::getOpenFileName(this, "Открыть"));
    if(!file)
    {
        QMessageBox::critical(this, "Ошибка", "Не удалось открыть файл");
        return;
    }
    file->open(QIODevice::ReadOnly);
    Tab newTab;
    QWidget *tabWidget = new QWidget(m_tabWidget);
    QGridLayout *tabLayout = new QGridLayout;
    QTextEdit *tabEdit = new QTextEdit;
    tabEdit->setText(file->readAll());
    tabLayout->setMargin(0);
    tabLayout->addWidget(tabEdit);
    tabWidget->setLayout(tabLayout);
    file->close();

    newTab.m_file = file;
    newTab.m_textEdit = tabEdit;
    m_tabs.push_back(newTab);
    *m_list << QFileInfo(*file).fileName();
    m_listModel->setStringList(*m_list);
    m_tabWidget->addTab(tabWidget, QFileInfo(*file).fileName());
    m_tabWidget->setCurrentIndex(m_tabWidget->count() - 1);
    toggleActions(true);

    if(QFileInfo(*file).suffix() == "js")
        new Highlighter(new QFile(":/hl.xml"), "js", tabEdit->document());
    else if(QFileInfo(*file).suffix().contains(QRegExp("^(h|hpp|c|cpp)$")))
        new Highlighter(new QFile(":/hl.xml"), "cpp", tabEdit->document());


    connect(tabEdit, SIGNAL(textChanged()), SLOT(slotChanged()));
    connect(m_viewerListView, SIGNAL(clicked(QModelIndex)), SLOT(slotSelectTab(QModelIndex)));
    newTab.m_saved = true;

}

void Editor::slotNew()
{
    Tab newTab;
    QWidget *tabWidget = new QWidget(m_tabWidget);
    QGridLayout *tabLayout = new QGridLayout;
    QTextEdit *tabEdit = new QTextEdit;
    tabLayout->setMargin(0);
    tabLayout->addWidget(tabEdit);
    tabWidget->setLayout(tabLayout);


    newTab.m_file = nullptr;
    newTab.m_saved = true;
    newTab.m_textEdit = tabEdit;
    m_tabs.push_back(newTab);
    m_tabWidget->addTab(tabWidget, QString("Новый документ"));
    m_tabWidget->setCurrentIndex(m_tabWidget->count() - 1);

    connect(tabEdit, SIGNAL(textChanged()), SLOT(slotChanged()));
    toggleActions(true);
    *m_list << QString("Новый документ");
    m_listModel->setStringList(*m_list);

    connect(m_viewerListView, SIGNAL(clicked(QModelIndex)), SLOT(slotSelectTab(QModelIndex)));
}

void Editor::slotSave()
{
    Tab &tab = m_tabs[m_tabWidget->currentIndex()];
    if(!tab.m_file)
    {
        slotSaveAs();
        return;
    }
    tab.m_file->open(QIODevice::ReadWrite | QIODevice::Truncate | QIODevice::Text);
    if(QFileInfo(*tab.m_file).suffix() == "pdf")
    {
        QPrinter printer(QPrinter::PrinterResolution);
        printer.setOutputFormat(QPrinter::PdfFormat);
        printer.setPaperSize(QPrinter::A4);
        printer.setOutputFileName(tab.m_file->fileName());

        QTextDocument doc;
        doc.setPlainText(tab.m_textEdit->toPlainText());
        doc.setPageSize(printer.pageRect().size());
        doc.print(&printer);
    }
    else
        tab.m_file->write(tab.m_textEdit->toPlainText().toUtf8());
    m_tabWidget->setTabText(m_tabWidget->currentIndex(), QFileInfo(*(m_tabs[m_tabWidget->currentIndex()].m_file)).fileName());
    tab.m_file->close();
}

void Editor::slotSaveAs()
{
    QString path(QFileDialog::getSaveFileName(this, "Сохранить как..."));
    if(path == "")
    {
        QMessageBox::critical(this, "Ошибка", "Не удалось сохранить файл");
        bypass = true;
        return;
    }
    QFile *file = new QFile(path);
    m_tabs[m_tabWidget->currentIndex()].m_file = file;
    m_tabWidget->setTabText(m_tabWidget->currentIndex(),  QFileInfo(*file).fileName());
    //m_listModel->setData(m_listModel->index(m_tabWidget->currentIndex()), QFileInfo(*file).fileName());
    m_list->replace(m_tabWidget->currentIndex(), QFileInfo(*file).fileName());
    m_listModel->setStringList(*m_list);
    slotSave();
}

void Editor::slotSaveAll()
{
    for(int i = 0; i < m_tabWidget->count(); i++)
        if(!m_tabs[i].m_saved)
        {
            m_tabWidget->setCurrentIndex(i);
            slotSave();
        }
}

void Editor::slotClose()
{
    slotCloseTab(m_tabWidget->currentIndex());
}

void Editor::slotCloseAll()
{
    slotCloseTab(0);
    if(m_tabWidget->count() != 0)
        slotCloseAll();

    toggleActions(false);
}

void Editor::slotQuit()
{
    QApplication::postEvent(this, new QCloseEvent());
    close();
}

void Editor::slotCut()
{
    m_tabs[m_tabWidget->currentIndex()].m_textEdit->cut();
}

void Editor::slotCopy()
{
    m_tabs[m_tabWidget->currentIndex()].m_textEdit->copy();
}

void Editor::slotPase()
{
    m_tabs[m_tabWidget->currentIndex()].m_textEdit->paste();
}

void Editor::slotDelete()
{
    m_tabs[m_tabWidget->currentIndex()].m_textEdit->insertPlainText("");
}

void Editor::slotSelectAll()
{

    m_tabs[m_tabWidget->currentIndex()].m_textEdit->selectAll();
}

void Editor::slotViewer(bool toggle)
{
    m_viewerDock->setVisible(toggle);
}

void Editor::slotExplorer(bool toggle)
{
    m_explorerDock->setVisible(toggle);
}

void Editor::slotAbout()
{
    QMessageBox::about(this, "О программе", "Лабораторная работа по С++");
}

void Editor::slotChanged()
{
    int i = m_tabWidget->currentIndex();
    Tab &tab = m_tabs[i];

    if(tab.m_saved)
        m_tabWidget->setTabText(i, m_tabs[i].m_file ? QFileInfo(*(m_tabs[i].m_file)).fileName() + " *" : "Новый документ *");
    tab.m_saved = false;
}

void Editor::slotCloseTab(int index)
{
    m_tabWidget->setCurrentIndex(index);
    /*if(!m_tabs[index].m_saved)
    {
        if(QMessageBox::Yes == QMessageBox::question(this, "Сохранить документ", "Сохранить " + m_tabWidget->tabText(index) + "?", QMessageBox::Yes, QMessageBox::No))
            slotSave();
        else
            return;
    }*/

    if(!m_tabs[index].m_saved)
    {
        auto answer = QMessageBox::question(this, "Закрыть программу", "Сохранить догументы?", QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);

        if (QMessageBox::Save == answer)
            slotSave();

        if (QMessageBox::Cancel == answer)
            return;
    }

    m_tabs.remove(index);
    m_list->removeAt(index);
    m_listModel->setStringList(*m_list);
    m_tabWidget->removeTab(index);

    if(!m_tabs.count())
        toggleActions(false);
}

void Editor::toggleActions(bool flag)
{
    m_saveAction->setEnabled(flag);
    m_saveAsAction->setEnabled(flag);
    m_saveAllAction->setEnabled(flag);
    m_closeAction->setEnabled(flag);
    m_closeAllAction->setEnabled(flag);
    m_cutAction->setEnabled(flag);
    m_copyAction->setEnabled(flag);
    m_pasteAction->setEnabled(flag);
    m_deleteAction->setEnabled(flag);
    m_selectAllAction->setEnabled(flag);
}

void Editor::slotExplorerOpen(QModelIndex index)
{
    QFileInfo file = m_fileModel->fileInfo(index);
    if(file.isDir())
        return;
    for(int i = 0; i < m_tabs.count(); i++)
        if(file.filePath() == m_tabs[i].m_file->fileName())
            return;

    QFile *new_file = new QFile(file.filePath());
    if(!new_file)
    {
        QMessageBox::critical(this, "Ошибка", "Не удалось открыть файл");
        return;
    }
    new_file->open(QIODevice::ReadOnly);
    Tab newTab;
    QWidget *tabWidget = new QWidget(m_tabWidget);
    QGridLayout *tabLayout = new QGridLayout;
    QTextEdit *tabEdit = new QTextEdit;
    tabEdit->setText(new_file->readAll());
    tabLayout->setMargin(0);
    tabLayout->addWidget(tabEdit);
    tabWidget->setLayout(tabLayout);
    new_file->close();

    newTab.m_file = new_file;
    newTab.m_saved = true;
    newTab.m_textEdit = tabEdit;
    m_tabs.push_back(newTab);
    *m_list << QFileInfo(*new_file).fileName();
    m_listModel->setStringList(*m_list);
    m_tabWidget->addTab(tabWidget, QFileInfo(*new_file).fileName());
    toggleActions(true);
}

void Editor::slotSelectTab(QModelIndex index)
{
    QString name = m_listModel->data(index).toString();
    int i = m_list->indexOf(QRegExp("^" + name + "$"));
    m_tabWidget->setCurrentIndex(index.row());
}
